# 
# This program is free software: you can redistribute it and/or modify it under 
# the terms of the GNU General Public License as published by 
# the Free Software Foundation, either version 3 of the License, 
# or (at your option) any later version.
# 
# 
# This program is distributed in the hope that it will be useful, 
# but WITHOUT ANY WARRANTY; without even the implied warranty of 
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
# See the GNU General Public License for more details.
# 
# 
# You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <https://www.gnu.org/licenses/>.
# 
# Copyright © 2019 Cloud Linux Software Inc.
# 
# This software is also available under ImunifyAV commercial license,
# see <https://www.imunify360.com/legal/eula>

package Imunify::Render;

use strict;
use warnings FATAL => 'all';

use Cpanel::JSON;
use Imunify::File;
use Imunify::Utils;
use Imunify::Config;
use Whostmgr::HTMLInterface ();

use constant MODE => 'prod';

use constant  {
    HTTP_STATUS_OK => 200,
    HTTP_STATUS_INTERNAL_ERROR => 500,
    HTTP_STATUS_BAD_GATEWAY => 502
};

#use CGI::Carp qw(fatalsToBrowser); # uncomment to debug 500 error

my %HTTP_STATUS = (
    &HTTP_STATUS_OK => 'HTTP/1.1 200 OK',
    &HTTP_STATUS_INTERNAL_ERROR => 'HTTP/1.1 500 Internal Server Error',
    &HTTP_STATUS_BAD_GATEWAY => 'HTTP/1.1 502 Bad Gateway'
);

sub JSONHeader {
    my ($status, $command) = @_;
    $command = '' if !defined $command;
    $command =~ s/[^[:alnum:][:punct:] ]+?/ /g;
    $status = HTTP_STATUS_OK if !defined $status || !defined $HTTP_STATUS{$status};

    print "X-I360-COMMAND: $command\n" if MODE ne 'prod';
    print $HTTP_STATUS{$status}, "\n";
    print "Content-type: application/json; charset=utf-8\n\n";
}

sub JSON {
    my ($data, $command, $warnings) = @_;
    my %result = (
        'result' => 'success',
        'data' => $data
    );

    if ($warnings) {
        %result = (
            'result' => 'warnings',
            'messages' => $warnings,
            'data' => $data,
        );
    }

    JSONHeader(HTTP_STATUS_OK, $command);
    print Cpanel::JSON::SafeDump(\%result);
    exit 0;
}

sub escapeParams {
    my ($value) = @_;
    $value =~ s/'/'\\''/g;
    $value = "'".$value."'";
    return $value;
}

sub assets {
    my ($type) = @_;
    my $cssAction = \&{$type . 'Css'};
    my $jsAction = \&{$type . 'Js'};
    my $manifest = Imunify::Utils::getManifest($type);

    my @js = (
        'js/config.js', # if you changed config.js, add `?v=1`
        "static/$manifest->{'polyfills.js'}",
        "static/$manifest->{'vendor.js'}",
        "static/$manifest->{'main.js'}",
    );

    sub adminCss {
        Whostmgr::HTMLInterface::load_css(Imunify::Config->PLUGIN_PATH . '/assets/' . $_[0]);
    }

    sub clientCss {
        print '<link rel="stylesheet" href="assets/' . $_[0] . '" />';
    }

    sub adminJs {
        Whostmgr::HTMLInterface::load_js(Imunify::Config->PLUGIN_PATH . '/assets/' . $_[0]);
    }

    sub clientJs {
        print '<script src="assets/' . $_[0] . '" defer></script>';
    }

    foreach my $file (@js) {
        $jsAction->($file);
    }
}

1;
