# 
# This program is free software: you can redistribute it and/or modify it under 
# the terms of the GNU General Public License as published by 
# the Free Software Foundation, either version 3 of the License, 
# or (at your option) any later version.
# 
# 
# This program is distributed in the hope that it will be useful, 
# but WITHOUT ANY WARRANTY; without even the implied warranty of 
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
# See the GNU General Public License for more details.
# 
# 
# You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <https://www.gnu.org/licenses/>.
# 
# Copyright © 2019 Cloud Linux Software Inc.
# 
# This software is also available under ImunifyAV commercial license,
# see <https://www.imunify360.com/legal/eula>

package Imunify::Utils;



use strict;
use warnings FATAL => 'all';
use File::Temp;
use MIME::Base64;

use DBI;
use DBI qw(:sql_types);
use Data::Dumper;

use Cpanel::SafeRun::Errors ();
use Cpanel::JSON;
use Imunify::Render;
use Imunify::Config;

# use CGI::Carp qw(fatalsToBrowser); # uncomment to debug 500 error

sub execute {
    my ($action, $data, $isWHM) = @_;
    my $tmp = $isWHM ? File::Temp->new(DIR => '/var/imunify360/tmp') : File::Temp->new();
    $tmp->write(encode_base64(Cpanel::JSON::SafeDump($data), ''));
    $tmp->flush();
    my $command = join ' ', (Imunify::Config->COMMAND_WRAPPER_PATH, $action, $tmp->filename);
    return Cpanel::SafeRun::Errors::saferunallerrors(('/bin/sh', '-c', $command));
}

sub random {
    my ($len) = @_;
    my @chars = ('0'..'9', 'a'..'f');
    $len = 32 if !defined $len;
    my $string;

    while($len--){
        $string .= $chars[rand @chars];
    };

    return $string;
}

sub getPluginName {
    my $filename = '/etc/sysconfig/imunify360/cpanel/imunify-antivirus.conf';
    if(-e $filename && -f _ ){
       return 'ImunifyAV';
    }
    return 'imunify360';
}

sub getManifest {

    my $json = do {
        my $folder = $_[0] eq 'admin' ? 'cgi' : 'frontend/paper_lantern';
        my $file = $folder . '/imunify/assets/static/manifest.json';
        open(my $json_fh, '<:encoding(UTF-8)', $file)
            or die("Can't open manifest.json: $!\n");
        local $/;
        <$json_fh>
    };
    return Cpanel::JSON::Load($json);
}

sub dump {
    my ($data) = @_;
    Imunify::Render::JSONHeader(Imunify::Render->HTTP_STATUS_OK, $data);
    print Dumper($data);
    exit 0;
}

sub escapeParams {
    my ($value) = @_;
    $value =~ s/'/'\\''/g;
    $value = "'".$value."'";
    return $value;
}

1;
